/*
 * Sphere.cpp
 *
 *  Created on: Aug 10, 2012
 *      Author: scottyoung
 */

#include "Sphere.h"
#include <math.h>

Sphere::Sphere(Vec3f c, float r, Vec3f m) {
	center = c;
	radius = r;
	material = new Material(m);
}

Sphere::~Sphere() {

}

bool Sphere::intersect(const Ray &r, Hit &h, float tMin)
{
	bool hit = false;

	Vec3f R = r.getOrigin() - center;
	Vec3f D = r.getDirection();
	Vec3f N;

	//R.Write(); cout << " "; D.Write();

	float b = R.Dot3(D);
	float c = R.Dot3(R) - radius*radius;

	float discriminant = b*b-c;
	float hit1, hit2, tValue;

	if(discriminant >= 0)
	{
		//Find the closest intersection
		hit1 = -b  +  sqrt(discriminant);
		hit2 = -b  -  sqrt(discriminant);
		tValue = min(hit1, hit2);

		//Find the normal
		N = R+D*tValue;
		N.Normalize();

		if(tValue > tMin && tValue < h.getT())
		{
			h.set(tValue, material, N, r);
			hit = true;
		}
	}

	return(hit);
}

void Sphere::print(){
	cout << "Sphere: pos @(" << center.x() <<","<< center.y()<<","<<center.z()<<") radius = "<<radius
			<< " color: (" << material->color.r() << "," << material->color.g() << "," << material->color.b() << ")" << endl;
}

