#ifndef _SceneParser_H_
#define _SceneParser_H_

#include "vectors.h"

#include "Object3D.h"
#include "Camera.h"
#include "OrthographicCamera.h"
#include "Group.h"
#include "Sphere.h"

#define MAX_PARSER_TOKEN_LENGTH 100

// ====================================================================
// ====================================================================

class SceneParser {

public:

  // CONSTRUCTORS, DESTRUCTOR & INITIALIZE
  SceneParser();
  SceneParser(const char *filename);
  ~SceneParser();

  // ACCESSORS
  Group* getGroup() { return group; }
  Camera* getCamera() { return camera; }
  Vec3f getBackgroundColor() { return background_color; }

private:

  // HELPER FUNCTIONS
  void initialize();
  int getToken(char token[MAX_PARSER_TOKEN_LENGTH]);
  void parseFile();
  Object3D* parseObject(char token[MAX_PARSER_TOKEN_LENGTH]);
  Group* parseGroup();
  Camera* parseOrthographicCamera();
  void parseBackground();
  void parseMaterial();
  Sphere* parseSphere();
  Vec3f readVec3f();
  float readFloat();
  int readInt();
  
  // REPRESENTATION
  Group *group;
  Camera *camera;
  Vec3f background_color;
  Vec3f ambient_light;

  Vec3f current_object_color;
  FILE *file;

};

// ====================================================================
// ====================================================================

#endif
