#ifndef _LIGHT_H_
#define _LIGHT_H_

#include "vectors.h"

// ====================================================================
// ====================================================================

class Light {

public:

  // CONSTRUCTOR & DESTRUCTOR
  Light() {}
  virtual ~Light() {}

  // VIRTUAL METHOD
  virtual void getIllumination (const Vec3f &p, Vec3f &dir, Vec3f &col, float &distance) const = 0;
  virtual void print()=0;

protected:
  Vec3f color;

};

// ====================================================================
// ====================================================================

class DirectionalLight : public Light {

public:

  // CONSTRUCTOR & DESTRUCTOR
  DirectionalLight() { 
    direction = Vec3f(0,0,0);
    color = Vec3f(1,1,1); }
  DirectionalLight(const Vec3f &d, const Vec3f &c) {
    direction = d; direction.Normalize();
    color = c; }
  virtual ~DirectionalLight() {}

  // VIRTUAL METHOD
  void getIllumination (const Vec3f &p, Vec3f &dir, Vec3f &col, float &distance) const {
    // the direction to the light is the opposite of the
    // direction of the directional light source
    dir = direction * (-1.0f);
    distance = INFINITY;
    col = color; }

  void print(){
	  cout << "Directional Light: dir=(" << direction.x() << "," << direction.y() << "," << direction.z() << ")"
			<< " color=(" << color.r() << "," << color.g() << "," << color.b() << ")" << endl;
  }


private:

  // REPRESENTATION
  Vec3f direction;

};

// ====================================================================
class PointLight : public Light{

public:
	  PointLight(const Vec3f &p, const Vec3f &c){
		  position = p;
		  color = c;
	  }
	  virtual ~PointLight() {}
	  void getIllumination (const Vec3f &p, Vec3f &dir, Vec3f &col, float &distance) const {
	    dir = position-p;
	    distance = dir.Length();
	    dir.Normalize();
	    col = color; }

	  void print(){
		  cout << "Point Light: "
				<< " pos=(" << position.x() << "," << position.y() << "," << position.z() << ")"
				<< " color=(" << color.r() << "," << color.g() << "," << color.b() << ")" << endl;
	  }
private:

	  Vec3f position;

};

// ====================================================================


#endif
