/*
 * Plane.cpp
 *
 *  Created on: Aug 14, 2012
 *      Author: scottyoung
 */

#include "Plane.h"

#define EPSILON .0001
#define BIG 1000

Plane::Plane(Vec3f n, float o, Material *m) {
	normal = n;
	normal.Normalize();
	offset = o;
	material = m;
	isGroup=false;
	isTri=false;
}

Plane::~Plane() {

}

bool Plane::intersect(const Ray &r, Hit &h, float tMin)
{
	bool isHit = false;
	Vec3f N = normal;
	Vec3f P = r.getOrigin();
	Vec3f D = r.getDirection();
	float a = offset;

	//Does (N * D) != 0? (we have a hit)
	if(fabs(N.Dot3(D)) > .0001)
	{
		float tValue = (a - N.Dot3(P))/(N.Dot3(D));
		if(tValue > tMin && tValue < h.getT())
		{
			h.set(tValue, material, N, r);
			isHit = true;
		}
	}

	RayTracingStats::IncrementNumIntersections();

	return(isHit);
}

void Plane::setBounds(){
	//only if the plane is perfectly axis-aligned does the bounding box have non-infinite parameters
	float xBounds = BIG, yBounds = BIG, zBounds = BIG;
	if(fabs(normal.x())-1.0 < EPSILON)
		xBounds = 0;
	else if(fabs(normal.y())-1.0 < EPSILON)
		yBounds = 0;
	else if(fabs(normal.z())-1.0 < EPSILON)
		zBounds = 0;
	bounds = BoundingBox(Vec3f(-xBounds,-yBounds,-zBounds), Vec3f(xBounds,yBounds,zBounds));
}

void Plane::print(){
	cout << "Plane: offset =" << offset <<" normal = "
			<< normal.x() <<","<< normal.y()<<","<<normal.z() << endl;
}

void Plane::paint(){
	Vec3f v = Vec3f(1,0,0);
	//If the normal is the x-axis, choose the y-axis as V
	if((normal-v).Length() < EPSILON)
		v = Vec3f(0,1,0);

	Vec3f basis1, basis2;
	Vec3f::Cross3(basis1, normal, v);
	Vec3f::Cross3(basis2, normal, basis1);

	Vec3f tr, tl, br, bl;

	tr = Vec3f(basis1.x()*BIG+basis2.x()*BIG, basis1.y()*BIG+basis2.y()*BIG, basis1.z()*BIG+basis2.z()*BIG);
	tl = Vec3f(-basis1.x()*BIG+basis2.x()*BIG, -basis1.y()*BIG+basis2.y()*BIG, -basis1.z()*BIG+basis2.z()*BIG);
	br = Vec3f(basis1.x()*BIG-basis2.x()*BIG, basis1.y()*BIG-basis2.y()*BIG, basis1.z()*BIG-basis2.z()*BIG);
	bl = Vec3f(-basis1.x()*BIG-basis2.x()*BIG, -basis1.y()*BIG-basis2.y()*BIG, -basis1.z()*BIG-basis2.z()*BIG);

	tr += normal * offset;
	tl += normal * offset;
	br += normal * offset;
	bl += normal * offset;

	material->glSetMaterial();
	glBegin(GL_TRIANGLES);
		glVertex3f(tr.x(), tr.y(), tr.z());
		glVertex3f(tl.x(), tl.y(), tl.z());
		glVertex3f(br.x(), br.y(), br.z());

		glVertex3f(bl.x(), bl.y(), bl.z());
		glVertex3f(br.x(), br.y(), br.z());
		glVertex3f(tl.x(), tl.y(), tl.z());

		//Put triangles on the underside too
		glVertex3f(tr.x(), tr.y(), tr.z());
		glVertex3f(br.x(), br.y(), br.z());
		glVertex3f(tl.x(), tl.y(), tl.z());

		glVertex3f(bl.x(), bl.y(), bl.z());
		glVertex3f(tl.x(), tl.y(), tl.z());
		glVertex3f(br.x(), br.y(), br.z());

	glEnd();

}
