/*
 * Transform.cpp
 *
 *  Created on: Aug 14, 2012
 *      Author: scottyoung
 */

#include "Transform.h"

Transform::Transform(Matrix m, Object3D *o) {
	mat = m;
	obj = o;
	isGroup=false;
	isTri=false;
}

Transform::~Transform() {
	// TODO Auto-generated destructor stub
}

bool Transform::intersect(const Ray &r, Hit &h, float tMin){

	bool hit = false;
	Vec3f D = r.getDirection();
	Vec3f O = r.getOrigin();
	Matrix invM(mat);
	invM.Inverse();

	invM.TransformDirection(D);
	invM.Transform(O);

	Ray R(D, O);
	hit = obj->intersect(R, h, tMin);
	if(hit)
	{
		Vec3f N = h.getNormal();
		Matrix invMT(invM);
		invMT.Transpose();

		invMT.TransformDirection(N);
		N.Normalize();
		h.set(h.getT(), h.getMaterial(), N, r);
	}

	return(hit);
}

/**
 * Bounding box for a transform. Note this does *NOT* implement the
 * triangle bounding box speedup (treats objects in abstract)
 */
void Transform::setBounds(){
	obj->setBounds();
	BoundingBox b = obj->getBoundingBox();
	//First, get all the corners of the box
	Vec3f p1(b.getMin());									  //mmm
	Vec3f p2(b.getMin().x(), b.getMin().y(), b.getMax().z()); //mmM
	Vec3f p3(b.getMin().x(), b.getMax().y(), b.getMax().z()); //mMM
	Vec3f p4(b.getMin().x(), b.getMax().y(), b.getMin().z()); //mMm
	Vec3f p5(b.getMax()); 									  //MMM
	Vec3f p6(b.getMax().x(), b.getMax().y(), b.getMin().z()); //MMm
	Vec3f p7(b.getMax().x(), b.getMin().y(), b.getMin().z()); //Mmm
	Vec3f p8(b.getMax().x(), b.getMin().y(), b.getMax().z()); //MmM
	//Now transform them
	mat.Transform(p1);
	mat.Transform(p2);
	mat.Transform(p3);
	mat.Transform(p4);
	mat.Transform(p5);
	mat.Transform(p6);
	mat.Transform(p7);
	mat.Transform(p8);
	//Now build up the transformed object:
	bounds = BoundingBox(p1, p1);
	bounds.includePoint(p2);
	bounds.includePoint(p3);
	bounds.includePoint(p4);
	bounds.includePoint(p5);
	bounds.includePoint(p5);
	bounds.includePoint(p6);
	bounds.includePoint(p7);
	bounds.includePoint(p8);
}

void Transform::print(){
	cout << "Transform: " << endl;
	mat.Write();
	obj->print();
}

void Transform::paint(){
	glPushMatrix();
	GLfloat *glMatrix = mat.glGet();
	glMultMatrixf(glMatrix);
	delete[] glMatrix;
	//Now paint the child
	obj->paint();
	glPopMatrix();
}
